@@ -39,12 +39,15 @@
 #include <config.h>
 
 #include "gtkfilesystemgnomevfs.h"
+#include <gtk/gtkversion.h>
 
 #include <libgnomevfs/gnome-vfs.h>
 #include <libgnomevfs/gnome-vfs-volume-monitor.h>
 #include <libgnomevfs/gnome-vfs-drive.h>
 #include <glib/gi18n-lib.h>
 
+#include <glib/gstdio.h>
+
 #ifdef USE_GCONF
 #include <gconf/gconf-client.h>
 #endif
@@ -67,6 +70,13 @@ typedef struct _GtkFileSystemGnomeVFSClass GtkFileSystemGnomeVFSClass;
 #define GTK_IS_FILE_SYSTEM_GNOME_VFS_CLASS(klass)  (G_TYPE_CHECK_CLASS_TYPE ((klass), GTK_TYPE_FILE_SYSTEM_GNOME_VFS))
 #define GTK_FILE_SYSTEM_GNOME_VFS_GET_CLASS(obj)   (G_TYPE_INSTANCE_GET_CLASS ((obj), GTK_TYPE_FILE_SYSTEM_GNOME_VFS, GtkFileSystemGnomeVFSClass))
 
+#ifdef G_OS_WIN32
+#define gnome_authentication_manager_push_async()
+#define gnome_authentication_manager_pop_async()
+#define gnome_authentication_manager_push_sync()
+#define gnome_authentication_manager_pop_sync()
+#endif
+
 static GType type_gtk_file_system_gnome_vfs = 0;
 static GType type_gtk_file_folder_gnome_vfs = 0;
 
@@ -237,6 +247,11 @@ static gboolean gtk_file_system_gnome_vfs_remove_bookmark (GtkFileSystem     *fi
 							   const GtkFilePath *path,
 							   GError           **error);
 static GSList * gtk_file_system_gnome_vfs_list_bookmarks  (GtkFileSystem *file_system);
+static gchar  * gtk_file_system_gnome_vfs_get_bookmark_label (GtkFileSystem     *file_system,
+							      const GtkFilePath *path);
+static void     gtk_file_system_gnome_vfs_set_bookmark_label (GtkFileSystem     *file_system,
+							      const GtkFilePath *path,
+							      const gchar       *label);
 
 static GType gtk_file_folder_gnome_vfs_get_type   (void);
 static void  gtk_file_folder_gnome_vfs_class_init (GtkFileFolderGnomeVFSClass *class);
@@ -376,6 +391,10 @@ gtk_file_system_gnome_vfs_iface_init (GtkFileSystemIface *iface)
   iface->insert_bookmark = gtk_file_system_gnome_vfs_insert_bookmark;
   iface->remove_bookmark = gtk_file_system_gnome_vfs_remove_bookmark;
   iface->list_bookmarks = gtk_file_system_gnome_vfs_list_bookmarks;
+#if GTK_CHECK_VERSION(2,7,0)
+  iface->get_bookmark_label = gtk_file_system_gnome_vfs_get_bookmark_label;
+  iface->set_bookmark_label = gtk_file_system_gnome_vfs_set_bookmark_label;
+#endif
 }
 
 static void
@@ -921,9 +940,32 @@ gtk_file_system_gnome_vfs_volume_get_base_path (GtkFileSystem        *file_syste
 {
   char *uri;
 
-  if (GNOME_IS_VFS_DRIVE (volume))
-    uri = gnome_vfs_drive_get_activation_uri (GNOME_VFS_DRIVE (volume));
-  else if (GNOME_IS_VFS_VOLUME (volume))
+  if (GNOME_IS_VFS_DRIVE (volume)) {
+    GnomeVFSVolume *vfs_volume;
+
+    /* So... the drive may have grown a volume (from mounting it from
+     * the file chooser) which may have a different activation_uri
+     * than the drive since it may be mounted anywhere. In fact, the
+     * activation_uri is in the drive is only a guess (from
+     * e.g. /etc/fstab) at best.. at worst, the activation_uri in the
+     * drive may be completely bogus (if the drive was picked up from
+     * something different than e.g. /etc/fstab)..
+     *
+     * Quick solution here is to check if the drive has got a volume,
+     * and if so, use that activation_uri.
+     */
+
+    vfs_volume = gnome_vfs_drive_get_mounted_volume (GNOME_VFS_DRIVE (volume));
+    if (vfs_volume != NULL)
+      {
+        uri = gnome_vfs_volume_get_activation_uri (vfs_volume);
+	gnome_vfs_volume_unref (vfs_volume);
+      } 
+    else
+      {
+        uri = gnome_vfs_drive_get_activation_uri (GNOME_VFS_DRIVE (volume));
+      }
+  } else if (GNOME_IS_VFS_VOLUME (volume))
     uri = gnome_vfs_volume_get_activation_uri (GNOME_VFS_VOLUME (volume));
   else
     {
@@ -1424,11 +1466,11 @@ gtk_file_system_gnome_vfs_parse (GtkFileSystem     *file_system,
 
       if (filesystem_path)
 	{
-	  switch (filesystem_path[0])
-	    {
-	    case '/':
+	  if (g_path_is_absolute (filesystem_path))
 	      uri = gnome_vfs_get_uri_from_local_path (filesystem_path);
-	      break;
+	  else switch (filesystem_path[0])
+	    {
+#ifndef G_OS_WIN32
 	    case '~':
 	      /* deliberately falling into default case on fail */
 	      path = gnome_vfs_expand_initial_tilde (filesystem_path);
@@ -1440,6 +1482,7 @@ gtk_file_system_gnome_vfs_parse (GtkFileSystem     *file_system,
 		}
 	      g_free (path);
 	      /* don't insert break here, read above comment */
+#endif
 	    default:
 	      escaped = gnome_vfs_escape_path_string (filesystem_path);
 	      base_dir = g_strconcat (base_uri, "/", NULL);
@@ -1719,7 +1762,7 @@ bookmark_list_write (GSList *bookmarks, GError **error)
 	  goto io_error;
 	}
 
-      if (rename (tmp_filename, filename) == -1)
+      if (g_rename (tmp_filename, filename) == -1)
 	{
 	  saved_errno = errno;
 	  goto io_error;
@@ -1746,7 +1789,7 @@ bookmark_list_write (GSList *bookmarks, GError **error)
   result = FALSE;
 
   if (fd != -1)
-    unlink (tmp_filename); /* again, not much error checking we can do here */
+    g_unlink (tmp_filename); /* again, not much error checking we can do here */
 
  out:
 
@@ -1786,10 +1829,10 @@ gtk_file_system_gnome_vfs_insert_bookmark (GtkFileSystem     *file_system,
 
   for (l = bookmarks; l; l = l->next)
     {
-      char *bookmark, *space;
+      gchar *bookmark, *space;
 
       bookmark = l->data;
-      
+
       space = strchr (bookmark, ' ');
       if (space)
 	*space = '\0';
@@ -1843,9 +1886,9 @@ gtk_file_system_gnome_vfs_remove_bookmark (GtkFileSystem     *file_system,
 
   for (l = bookmarks; l; l = l->next)
     {
-      char *bookmark, *space;
+      gchar *bookmark, *space;
 
-      bookmark = (char *)l->data;
+      bookmark = l->data;
       space = strchr (bookmark, ' ');
       if (space)
 	*space = '\0';
@@ -1899,14 +1942,13 @@ gtk_file_system_gnome_vfs_list_bookmarks (GtkFileSystem *file_system)
 
   for (l = bookmarks; l; l = l->next)
     {
-       char *bookmark, *space;
-  
-       bookmark = (char *)l->data;
-       space = strchr (bookmark, ' ');
-       if (space)
-	 *space = '\0';
-       
-       result = g_slist_prepend (result, gtk_file_system_uri_to_path (file_system, bookmark));
+      gchar *bookmark, *space;
+
+      bookmark = l->data;
+      space = strchr (bookmark, ' ');
+      if (space)
+	*space = '\0';
+      result = g_slist_prepend (result, gtk_file_system_uri_to_path (file_system, bookmark));
     }
 
   bookmark_list_free (bookmarks);
@@ -1915,6 +1957,92 @@ gtk_file_system_gnome_vfs_list_bookmarks (GtkFileSystem *file_system)
   return result;
 }
 
+static gchar *
+gtk_file_system_gnome_vfs_get_bookmark_label (GtkFileSystem     *file_system,
+					      const GtkFilePath *path)
+{
+  GSList *bookmarks;
+  gchar *label;
+  GSList *l;
+  gchar *bookmark, *space, *uri;
+  
+  if (!bookmark_list_read (&bookmarks, NULL))
+    return NULL;
+
+  uri = gtk_file_system_path_to_uri (file_system, path);
+
+  label = NULL;
+  for (l = bookmarks; l && !label; l = l->next) 
+    {
+      bookmark = l->data;
+      space = strchr (bookmark, ' ');
+      if (!space)
+	continue;
+
+      *space = '\0';
+
+      if (strcmp (uri, bookmark) == 0)
+	label = g_strdup (space + 1);
+    }
+
+  g_free (uri);
+  bookmark_list_free (bookmarks);
+
+  return label;
+}
+
+static void
+gtk_file_system_gnome_vfs_set_bookmark_label (GtkFileSystem     *file_system,
+					      const GtkFilePath *path,
+					      const gchar       *label)
+{
+  GSList *bookmarks;
+  GSList *l;
+  gchar *bookmark, *space, *uri;
+  gboolean found;
+
+  if (!bookmark_list_read (&bookmarks, NULL))
+    return;
+
+  uri = gtk_file_system_path_to_uri (file_system, path);
+
+  found = FALSE;
+  for (l = bookmarks; l && !found; l = l->next) 
+    {
+      bookmark = l->data;
+      space = strchr (bookmark, ' ');
+      if (space)
+	*space = '\0';
+
+      if (strcmp (bookmark, uri) != 0)
+	{
+	  if (space)
+	    *space = ' ';
+	}
+      else
+	{
+	  g_free (bookmark);
+	  
+	  if (label && *label)
+	    l->data = g_strdup_printf ("%s %s", uri, label);
+	  else
+	    l->data = g_strdup (uri);
+
+	  found = TRUE;
+	  break;
+	}
+    }
+
+  if (found)
+    {
+      if (bookmark_list_write (bookmarks, NULL))
+	g_signal_emit_by_name (file_system, "bookmarks-changed", 0);
+    }
+  
+  g_free (uri);
+  bookmark_list_free (bookmarks);
+}
+
 #ifdef USE_GCONF
 static gboolean
 gtk_file_system_gnome_vfs_add_bookmark (GtkFileSystem     *file_system,
